### (c) TIG 2009-2011
### Adds new method to ArcCurve to change number of segments -
### like in the Entity Info dialog...
### Usage: arc.count_edges >>> 12
###        arc.set_segments= 36
###        arc.count_edges >>> 36
### It will NOT work on arcs with attached face(s) NOT 'coplanar' - 
### either with the arc or each other.
### Connected faces with orientation, materials, holes etc are retained.
### 20090522 First issue.
### 20090928 Added the ArcCurveTests, remakes Arc, Circle or Ngon.
### 20110216 Clashes with other methods fixed
### 20120121 Put inside TIG module and Plugins menu item added.
require 'sketchup.rb'
class Sketchup::ArcCurve
  def set_segments=(s)
    s=s.to_i; s=3 if s<3; s=4096 if s>4096
    model=Sketchup.active_model
    entities=model.active_entities
    center=self.center
    end_angle=self.end_angle
    normal=self.normal
    plane=self.plane
    radius=self.radius
    start_angle=self.start_angle
    xaxis=self.xaxis
    yaxis=self.yaxis
    count_edges=self.count_edges
    edges=self.edges
    faces=edges[0].faces
    return nil if s==count_edges ### check if it will change
    return nil if faces.length>2 ### check for 3D faces...
    qnormal=true if not faces[0]
    if faces.length==1
      qnormal=true if normal==faces[0].normal or normal==faces[0].normal.reverse!   
    end#if
    if faces.length==2
      qnormal=true if (normal==faces[0].normal or normal==faces[0].normal.reverse!) and (faces[0].normal==faces[1].normal or faces[0].normal==faces[1].normal.reverse!)
    end#if
    return nil if not qnormal
    ### NOW we know we have an arc with 0 or 1 or 2 coplanar face(s)
    faceNr=faces.length
    face1=faces[0]if faces[0];face2=faces[1]if faces[1]
    rev=true if face1 and face1.outer_loop.edges.include?(edges[0])
    mat1=face1.material if face1;mat2=face2.material if face2
    back1=face1.back_material if face1;back2=face2.back_material if face2
    norm1=face1.normal if face1;norm2=face2.normal if face2
    ### make new arc
    group=nil
    group=entities.add_group()
    ###====================================================================
    def is_loop?(arccurve)### circle OR polygon
      arccurve.vertices.each{|v|
        edgecount=0
        v.edges.each{|ee|edgecount+=1 if ee.curve==self}
        return false if edgecount<2 ### vert can have >1 edge BUT it might belong to other curve etc
      }
      return true
    end#if
    ###====================================================================
    if not is_loop?(self) ### == arc
      group=entities.add_group()
      arcEdges=group.entities.add_arc(center,xaxis,normal,radius,start_angle,end_angle,s)###
    else
      if self.is_polygon? ### > v7
        group=entities.add_group()
        arcEdges=group.entities.add_ngon(center,normal,radius,s)###
      else ### circle
        group=entities.add_group()
        arcEdges=group.entities.add_circle(center,normal,radius,s)###
      end#if
    end#if
    ### find any extg empty inner loops sharing arc's faces
    loops1=[];loops2=[]
    eloops1=[];eloops2=[]
    edges1=[];edges2=[]
    if face1 and face1.valid?
      loops1=(face1.loops.to_a-[face1.outer_loop])
      loops1.each{|e|eloops1.push(e)if not e.edges[0].faces[1]}
      eloops1.each{|e|edges1.push(e.edges)}
      edges1.flatten!
    end#if
    if face2 and face2.valid?
      loops2=(face2.loops.to_a-[face2.outer_loop])
      loops2.each{|e|eloops2.push(e)if not e.edges[0].faces[1]}
      eloops2.each{|e|edges2.push(e.edges)}
      edges2.flatten!
    end#if
    ### erase old arcc
    edges.each{|e|e.erase! if e.valid?}
    group.explode if group.valid? ### add in new arc
    arcEdges.each{|e|e.find_faces if e.valid?}if faceNr>0
    ### check if it's arc or circular 'inner' loop etc
    arcFaces=[]
    arcFaces=arcEdges[0].faces.to_a if arcEdges[0].valid?
    arcFaces.reverse! if faceNr==1 and not rev
    arcFaces.each{|e|e.erase! if e.valid?}if faceNr==0
    (arcFaces-[arcFaces[0]]).each{|e|e.erase! if e.valid?}if faceNr==1
    ### sort out empty inner loops
    edges1.each{|e|if e.valid?
      e.find_faces 
      e.faces.each{|face|face.erase! if not face.edges.include?(arcEdges[0])}
    end#if
    }
    edges2.each{|e|if e.valid?
      e.find_faces
      e.faces.each{|face|face.erase! if not face.edges.include?(arcEdges[0])}
    end#if
    }
    if arcFaces[0] and arcFaces[0].valid?
      arcFaces[0].material=mat1
      arcFaces[0].back_material=back1
      arcFaces[0].reverse! if arcFaces[0].normal != norm1
    end#if
    if arcFaces[1] and arcFaces[1].valid?
      arcFaces[1].material=mat2
      arcFaces[1].back_material=back2
      arcFaces[1].reverse! if arcFaces[1].normal != norm2
    end#if
  end#def
end#class
### Example = change all arcs/circles in selection to be NNN sided
### e.g. to smooth selected arcs in imported dxf plan...
### large selections with many segments might take a while to run...
### OR use menu item...
module TIG
def self.changearcsegments(segments)
  model=Sketchup.active_model
  name="修改弧线段数"
  if Sketchup.version.split('.')[0].to_i >= 7
	model.start_operation(name,true)
  else
	model.start_operation(name)
  end
  selection=model.selection
  arcs=[];selection.each{|e|
    if e.typename=="Edge" and e.curve and e.curve.typename=="ArcCurve"
      arcs.push(e.curve)if not arcs.include?(e.curve)
    end#if
  }
  ctr=1;arcs.each{|arc|
    Sketchup::set_status_text("Processing Arc "+ctr.to_s,SB_PROMPT)
    arc.set_segments= segments if arc.valid?
    ctr=ctr+1
  }
  Sketchup::set_status_text("",SB_PROMPT)
  model.commit_operation
  return nil
  ### returns number of arcs processed.
end#def
###
unless file_loaded?(File.basename(__FILE__))
  
file_loaded(File.basename(__FILE__))
path = Sketchup.find_support_file("Plugins/ArcCurveTests")
  ArcCurveTests_tb = UI::Toolbar.new("修改弧线段数-SUCJ.ME")
  cmd = UI::Command.new(("修改弧线段数_SUCJ.ME")) {
    segments=inputbox(["段数: "],[12],"修改弧线段数")
    self.changearcsegments(segments[0])if segments
  }
  cmd.small_icon = File.join(path,"icons/xghxds_24.png")
  cmd.large_icon = File.join(path,"icons/xghxds_36.png")
  cmd.tooltip = "修改弧线段数_SUCJ.ME"
  cmd.status_bar_text = "修改弧线段数_SUCJ.ME"
  ArcCurveTests_tb.add_item(cmd) 
  ArcCurveTests_tb.restore
  end
 file_loaded(File.basename(__FILE__))

end#TIG module

###